#define MPEGMAIN
#include <getopt.h>
#include "main.h"
#include "consts.h"
#include "cpptools.hh"


void SetMPEG2Mplex()
{
  /* multiplex stuff */
	cerr << "Setting up for MPEG2 mux" << endl;
	sectors_delay = 0;
	video_delay_ms = 180;
	audio_delay_ms = 180;
	audio1_delay_ms = 180;
	sector_size = 2048;
	packets_per_pack = 1;
	init_audio_buffer_size = 4;
	init_audio1_buffer_size = 4;
	init_video_buffer_size = 224;
	always_sys_header = FALSE;
	mplex_type = MPEG_MPEG2;
	mplex_pulldown_flag = PULLDOWN_AUTO;
	vcd_audio_pad = FALSE;
	user_mux_rate = 0;
	align_sequence_headers = 0;
	put_private2 = 0;
	frame_timestamps = TIMESTAMPS_ALL;
	VBR_multiplex = TRUE;
	use_computed_bitrate = COMPBITRATE_MAX;
	write_pec = 1;
	mux_SVCD_scan_offsets = 0;
	max_file_size = 0;
	mux_start_time = 0;
	mux_stop_time = 0;
	reset_clocks = 1;
	write_end_codes = 1;
	set_broken_link = 1;
}

void SetDVDMplex()
{
  /* multiplex stuff */
	cerr << "Setting up for DVD mux" << endl;
	sectors_delay = 0;
	video_delay_ms = 180;
	audio_delay_ms = 180;
	audio1_delay_ms = 180;
	sector_size = 2048;
	packets_per_pack = 1;
	init_audio_buffer_size = 4;
	init_audio1_buffer_size = 4;
	init_video_buffer_size = 232;
	always_sys_header = FALSE;
	use_computed_bitrate = COMPBITRATE_MAX;
	mplex_type = MPEG_DVD;
	mplex_pulldown_flag = PULLDOWN_AUTO;
	vcd_audio_pad = FALSE;
	user_mux_rate = 25200;
	align_sequence_headers = TRUE;
	put_private2 = TRUE;
	frame_timestamps = TIMESTAMPS_IONLY;
	VBR_multiplex = TRUE;
	write_pec = 1;
	mux_SVCD_scan_offsets = 0;
	max_file_size = 0;
	mux_start_time = 0;
	mux_stop_time = 0;
	reset_clocks = 0;
	write_end_codes = 0;
	set_broken_link = 0;
}

void SetSVCDMplex()
{
  /* multiplex stuff */
	cerr << "Setting up for SVCD mux" << endl;
	sectors_delay = 0;
	video_delay_ms = 180;
	audio_delay_ms = 180;
	audio1_delay_ms = 180;
	sector_size = SVCD_SECTOR_SIZE;
	packets_per_pack = 1;
	init_audio_buffer_size = 4;
	init_audio1_buffer_size = 4;
	init_video_buffer_size = 230;
	always_sys_header = FALSE;
	use_computed_bitrate = COMPBITRATE_MAX;
	mplex_type = MPEG_SVCD;
	mplex_pulldown_flag = PULLDOWN_AUTO;
	vcd_audio_pad = FALSE;
	user_mux_rate = 6972;
	align_sequence_headers = 1;
	put_private2 = 0;
	frame_timestamps = TIMESTAMPS_ALL;
	VBR_multiplex = TRUE;
	write_pec = 1;
	mux_SVCD_scan_offsets = 1;
	max_file_size = 0;
	mux_start_time = 0;
	mux_stop_time = 0;
	reset_clocks = 1;
	write_end_codes = 1;
	set_broken_link = 1;
}

void SetMPEG1Mplex()
{
  /* multiplex stuff */
	cerr << "Setting up for MPEG1 mux" << endl;
	sectors_delay = 0;
	video_delay_ms = 180;
	audio_delay_ms = 180;
	audio1_delay_ms = 180;
	sector_size = 2048;
	packets_per_pack = 1;
	init_audio_buffer_size = 4;
	init_audio1_buffer_size = 4;
	init_video_buffer_size = 46;
	always_sys_header = FALSE;
	mplex_type = MPEG_MPEG1;
	mplex_pulldown_flag = PULLDOWN_NONE;
	vcd_audio_pad = FALSE;
	user_mux_rate = 0;
	align_sequence_headers = 0;
	put_private2 = 0;
	frame_timestamps = TIMESTAMPS_ALL;
	VBR_multiplex = FALSE;
	use_computed_bitrate = COMPBITRATE_MAX;
	write_pec = 1;
	mux_SVCD_scan_offsets = 0;
	max_file_size = 0;
	mux_start_time = 0;
	mux_stop_time = 0;
	reset_clocks = 1;
	write_end_codes = 1;
	set_broken_link = 1;
}


void SetVCDMplex()
{
  /* multiplex stuff */
	cerr << "Setting up for VCD mux" << endl;
	sectors_delay = 400;
	video_delay_ms = 344;
	audio_delay_ms = 344;
	audio1_delay_ms = 344;
	sector_size = VIDEOCD_SECTOR_SIZE;
	packets_per_pack = 1;
	init_audio_buffer_size = 4;
	init_audio1_buffer_size = 4;
	init_video_buffer_size = 46;
	always_sys_header = FALSE;
	use_computed_bitrate = COMPBITRATE_NONE;
	mplex_type = MPEG_VCD;
	mplex_pulldown_flag = PULLDOWN_NONE;
	vcd_audio_pad = FALSE;
	user_mux_rate = 3486;
	align_sequence_headers = 0;
	put_private2 = 0;
	frame_timestamps = TIMESTAMPS_ALL;
	VBR_multiplex = 0;
	write_pec = 1;
	mux_SVCD_scan_offsets = 0;
	max_file_size = 0;
	mux_start_time = 0;
	mux_stop_time = 0;
	reset_clocks = 1;
	write_end_codes = 1;
	set_broken_link = 1;
}

void usage(char *progname)
{
	printf ("usage: %s [options] <input file>\n\n",progname);
	printf ("options:\n");
	printf ("  --help,             -h:  print help message\n");  
	printf ("  --type,             -t:  set output type (MPEG2, DVD, SVCD (default), MPEG1, VCD)\n");
	printf ("  --of,               -o:  set output file\n");
	printf ("  --audio_delay,      -a:  set audio delay in ms\n");
	printf ("  --video_delay,      -v:  set video delay in ms\n");
	exit(1);
}

int main(int argc, char **argv)
{
	int fda_out, fdv_out, fd_in=0;
	int64_t ptsdiff;
	int c;
	char *filename = NULL;
	char *type = "SVCD";
	int adelay = 0;
	int vdelay = 0;

	strcpy(AudioFilename,"tmp_as.mp2");
	strcpy(VideoFilename,"tmp_vs.m2v");
	
	while (1) {
		int option_index = 0;
		static struct option long_options[] =
		{{"type", required_argument, NULL, 't'},
		 {"audio_delay", required_argument, NULL, 'a'},
		 {"video_delay", required_argument, NULL, 'v'},
		 {"of",required_argument, NULL, 'o'},
		 {"help", no_argument , NULL, 'h'},
		 {0, 0, 0, 0}
		};
		c = getopt_long (argc, argv, "t:o:a:v:h",
				 long_options, &option_index);
		if (c == -1)
			break;
		
		switch (c) {
		case 't':
			type = optarg;
			break;
		case 'a':
			adelay = atoi(optarg);
			break;
		case 'v':
			vdelay = atoi(optarg);
			break;
		case 'o':
			filename = optarg;
			break;
		case 'h':
		case '?':
		default:
			usage(argv[0]);
		}
	}
	
	if (optind == argc-1)
		{
			if ((fd_in = open(argv[optind] ,O_RDONLY)) < 0){
				cerr << "Error opening input file " << argv[optind]<< endl;
				exit(1);
			}
			cerr << "Reading from " << argv[optind] << endl;
		} else {
			usage(argv[0]);
		}

	if (filename)
		strcpy(ProgramFilename,filename);
	else
		strcpy(ProgramFilename,"out.mpg");
	cerr << "Output File is: " << ProgramFilename << endl; 
		
	fda_out = open(AudioFilename,O_WRONLY|O_CREAT|O_TRUNC,
		       S_IRUSR|S_IWUSR);
	fdv_out = open(VideoFilename,O_WRONLY|O_CREAT|O_TRUNC,
		       S_IRUSR|S_IWUSR);
	if(fda_out < 0 || fdv_out < 0){
		cerr << "Error opening output files" << endl;
		exit(1);
	}

	if (!strcmp(type,"MPEG2"))
		SetMPEG2Mplex();
	else if (!strcmp(type,"DVD"))
		SetDVDMplex();
	else if (!strcmp(type,"SVCD"))
		SetSVCDMplex();
	else if (!strcmp(type,"MPEG1"))
		SetMPEG1Mplex();
	else if (!strcmp(type,"VCD"))
		SetVCDMplex();
	else usage(argv[0]);
	cerr << endl;

	cerr << "Opening " << AudioFilename << " for audio demux" << endl;
	cerr << "Opening " << VideoFilename << " for video demux" << endl;
	ptsdiff = es_demux(fd_in, fdv_out, fda_out);
	cerr << endl;

	if ( adelay || vdelay ) {
		ptsdiff = (vdelay - adelay)*90;
	}
	cerr << "VPTS - APTS = " << ptsdiff/90 << "ms" << endl;
	if (ptsdiff > 270000 || ptsdiff < -270000) {
	  cerr << "Warning pts difference seems too large."
	       << "Please set audio or video delay by hand" << endl;
	  exit(1);
	}

	if (ptsdiff > 0) video_delay_ms += ptsdiff/90;
	else audio_delay_ms += -ptsdiff/90;

	domplex(1,1);
	unlink(AudioFilename);
	unlink(VideoFilename);
	fprintf(stderr,"                                                            \n");
}
